#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

// One variable function 'f(x)'
class scalarfun_t {
public:
  virtual double eval(double x)=0;
};

// Two variables function 'f(x,y)'
class scalarfun2d_t {
public:
  virtual double eval(double x,double y)=0;
};

// Integral of a one var funct over an interval
class integral1d_t {
private:
  int N;                        // Nbr of integration pts
  double xa,xb;                 // integration interval
  scalarfun_t *fp;              // ptr to function
public:
  // Ctor, set values
  integral1d_t(int Na=0,double xaa=NAN,
               double xba=NAN,scalarfun_t *fpa=NULL) {
    set(Na,xaa,xba,fpa);
  }
  // Set values
  void set(int Na,double xaa,double xba,scalarfun_t *fpa) {
    N=Na; xa=xaa; xb=xba; fp=fpa;
  }
  // Computes the integral using Simpson's rule
  double integral() {  
    double
      h = (xb-xa)/N,
      h2 = h/2.0,
      finteg=0.0;
    for (int j=0; j<N; j++) {
      double x = j*h;
      finteg += 2.0*fp->eval(x);
      finteg += 4.0*fp->eval(x+h2);
    }
    finteg -= fp->eval(xa);
    finteg += fp->eval(xb);
    finteg *= h/6.0;
    return finteg;
  }
};

// Specific function to be integrated
class myfun_t : public scalarfun2d_t {
public:
  double eval(double x,double y) {
    return x*y;
    // return sin(M_PI*x)*sin(M_PI*y); // debe dar (2/pi)^2 = 0.40528
  }
} myfun;

// Integral of 2var functions over a rectangle
class integral2d_t {
private:
  scalarfun2d_t *f2dp;          // Ptr to 2var funct
  integral1d_t int1dy, int1dx;  // integrals over x and y
  // For a given x gives the function fx(y) = f(x,y)
  class fx_t : public scalarfun_t {
    double x;
    integral2d_t *i2dp;         // ptr to calling class
    friend class integral2d_t;
  public:
    double eval(double y) { return i2dp->f2dp->eval(x,y); }
  } fx;
  // For a given x, integral over y: g(x) = int_{y=a}^b f(x,y) dy
  class g_t : public scalarfun_t {
    integral2d_t *i2dp;         // ptr to calling class
    friend class integral2d_t;
  public:
    double eval(double x) {
      return i2dp->geval(x);
    }
  } g;
  double geval(double x) {
    fx.x = x;
    return int1dy.integral();
  }
  int N;
  double xa,xb,ya,yb;
  friend class fx_t;
  friend class g_t;
public:
  integral2d_t(int Na,
               double xaa,double xba,double yaa,double yba,
               scalarfun2d_t &f2d)
    : N(Na), xa(xaa), xb(xba), ya(yaa), yb(yba) {
    f2dp = &f2d;
    int1dy.set(N,ya,yb,&fx);
    int1dx.set(N,xa,xb,&g);
    g.i2dp = this;
    fx.i2dp = this;
  }
  double integral() {
    return int1dx.integral();
  }
};

int main() {
  int N=100;
  integral2d_t int2d(N,0,1,0,1,myfun);
  cout << "integral(x,y) = " << int2d.integral() << endl;
  return 0;
}
