#include <iostream>
#include <vector>
using namespace std;

class mathvec_t {
public:
  virtual int size()=0;
  virtual double operator[](int j) const =0;
  virtual void set(int j,double a)=0;
};

double sum(mathvec_t &v) {
  int N = v.size();
  double s = 0.0;
  for (int j=0; j<N; j++) s += v[j];
  return s;
}

class full_vec_t : public mathvec_t {
private:
  vector<double> v;
public:
  full_vec_t(int N,double a=0) : v(N,a) {}
  int size() { return v.size(); }
  double operator[](int j) const { return v[j]; }
  void set(int j,double a) { v[j] = a; }
};

class sp_vec_t : public mathvec_t {
private:
  int N;
  vector<int> indx;
  vector<double> v;
  int lower_bound(int j) const {
    int sz = indx.size(), k;
    for (k=0; k<sz; k++) if (indx[k]>=j) break;
    return k;
  }
public:
  sp_vec_t(int Na,double a=0) : N(Na) {
    if (a!=0.0) {
      for (int j=0; j<N; j++) set(j,a);
    }
  }
  int size() { return N; }
  double operator[](int j) const {
    int k = lower_bound(j);
    if (k<indx.size() && indx[k]==j) return v[k];
    else return 0;
  }
  void set(int j,double a) {
    int k = lower_bound(j);
    if (k<indx.size() && indx[k]==j) v[k]=a;
    else {
      indx.insert(indx.begin()+k,j);
      v.insert(v.begin()+k,a);
    }
  }
};

int main() {
  full_vec_t wf(10);
  sp_vec_t ws(10);
  for (int j=0; j<10; j+=2) {
    wf.set(j,j+3);
    ws.set(j,j+3);
  }
  cout << "sum(w), sparse " << sum(ws) << ", full " << sum(wf) << endl;
  return 0;
}
