#include <cmath>
#include <cstdio>
#include <cassert>
#include <cstdlib>
#include <iostream>
using namespace std;

class Function {
public:
  virtual double eval(double x) {};
};

double simpson(Function &f, double a, double b,int n) {
  double sum = 0.0, h=(b-a)/n;
  sum += (f.eval(a) + f.eval(b))*h/6.0;
  for (int j=0; j<n; j++) {
    double x = a+(j+0.5)*h;
    sum += f.eval(x)*h*(4.0/6.0);
  }
  for (int j=1; j<n; j++) {
    double x = a+j*h;
    sum += f.eval(x)*h*(2.0/6.0);
  }
  return sum;
}

class Linear : public Function {
public:
  double y0,m;
  double eval(double x) {
    return y0+m*x;
  }
};

class Sin : public Function {
public:
  double eval(double x) {
    return sin(x);
  }
};

int main() {
  Linear f;
  f.y0 = 0.0;
  f.m = 1.0;
  cout << "Integral of 'x' between 1 and 3 is " 
       << simpson(f, 1.0, 3.0, 2) << endl;

  Sin fsin;
  double sumex = cos(1.0)-cos(2.0);
  cout << "Integral of sin(x) between 0.0 and 2.0 is " 
       << simpson(fsin,1.0,2.0,10) << ", exact " << sumex << endl;
  return 0;
}
